;; Generated package description from kubed.el  -*- mode: lisp-data; no-byte-compile: t; lexical-binding:t -*-
(define-package "kubed" "0.7.0" "Kubernetes, Emacs, done!" '((emacs "29.1")) :commit "daf3bb9054bed987f8792ff227a3c6b060084834" :authors '(("Eshel Yaron" . "me@eshelyaron.com")) :maintainer '("Eshel Yaron" . "~eshel/kubed-devel@lists.sr.ht") :keywords '("tools" "kubernetes" "containers") :url "https://eshelyaron.com/kubed.html")
