;;; kubed-common.el --- Common definitons for Kubed   -*- lexical-binding: t; -*-

;; Copyright (C) 2024-2026  Free Software Foundation, Inc.

;; Author: Eshel Yaron <me@eshelyaron.com>
;; Keywords: tools

;;; Commentary:

;; This library provides definitions that both `kubed' and `kubed-tramp'
;; require.

;;; Code:

(defvar kubed-tramp-method "kubedv2"    ;Versioned, for compatibility.
  ;; (find-file "/kubedv2:CONTEXT%NAMESPACE%POD%CONTAINER:/some/file")
  "Name of the Kubed Tramp method.")

(defcustom kubed-kubectl-program "kubectl"
  "Name of `kubectl' executable to use for interacting with Kubernetes."
  :type 'string
  :group 'kubed)

(provide 'kubed-common)
;;; kubed-tramp.el ends here
